/*
 * 14/06/2002 - 20:50:39
 *
 * FileListExplorer version 1.1
 * Copyright (C) 2001 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.beans.filelistexplorer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

/** <TD>FileListExplorer est un JavaBean qui implmente un explorateur de fichiers cr  partir d'un composant JList.
 * La navigation se fait trs simplement  l'aide des touches flches et de la touche <I>enter</I> ou de la souris.<BR />Les vnements
 * suivant <CODE>fileChoosen</CODE> (double click ou touche enter sur un ficher), <CODE>fileSelected</CODE> (selection d'un fichier ou d'un rpertoire)
 * , <CODE>directoryChanged</CODE> (changement de rpertoire) sont envoys aux objets implantants l'interface <CODE>FileListExplorerListener</CODE> et
 * rpertoris dans la liste des couteurs du bean.</TD>
 * <BR /><BR /><IMG src='screen.jpg' hspace='30'>
 * @author Thierry Vouriot
 * @version 1.0
 */
public class FileListExplorer extends JList implements Serializable {
    
    /** Index dans le tableau des cones de l'URL de l'cone pour les fichiers
     */
    public final static int INDEX_FILE_ICON = 0;
    /** Index dans le tableau des cones de l'URL de l'cone pour les rpertoires
     */
    public final static int INDEX_DIRECTORY_ICON = 1;
    /** Index dans le tableau des cones de l'URL de l'cone pour l'lment actualiser (.)
     */
    public final static int INDEX_REFRESH_ICON = 2;
    /** Index dans le tableau des cones de l'URL de l'cone pour l'lment rpertoire parent (..)
     */
    public final static int INDEX_PARENT_ICON = 3;
    /** Index dans le tableau des cones de l'URL de l'cone pour les racines
     */
    public final static int INDEX_ROOT_ICON = 4;
    /** Tri par nom
     */
    public final static int SORT_BY_NAME = 0;
    /** Tri par extension
     */
    public final static int SORT_BY_EXTENSION = 1;
    /** Sparateur de dossier gnralement "/" ou "\"
     */
    public final static String FILE_SEPARATOR = System.getProperty("file.separator");
    /** <IMG src='file.gif'> URL de l'cone par dfaut pour les fichiers
     */
    public final URL DEFAULT_FILE_ICON = this.getClass().getResource("fle_img0.gif");
    /** <IMG src='directory.gif'> URL de l'cone par dfaut pour les rpertoires
     */
    public final URL DEFAULT_DIRECTORY_ICON = this.getClass().getResource("fle_img1.gif");
    /** <IMG src='refresh.gif'> URL de l'cone par dfaut de l'lment actualiser (.)
     */
    public final URL DEFAULT_REFRESH_ICON = this.getClass().getResource("fle_img2.gif");
    /** <IMG src='parent.gif'> URL de l'cone par dfaut de l'lment rpertoire parent (..)
     */
    public final URL DEFAULT_PARENT_ICON = this.getClass().getResource("fle_img3.gif");
    /** <IMG src='root.gif'> URL de l'cone par dfaut pour les racines (A:,C:,/,...)
     */
    public final URL DEFAULT_ROOT_ICON = this.getClass().getResource("fle_img4.gif");
    
    /** Model contenant les lments de la liste
     */
    private final class FileListModel extends DefaultListModel implements Serializable {
        
        /** Fonction qui  partir d'un tableau de fichiers met  jour la liste
         * @param tab tableau des fichiers  afficher dans la liste
         * @param roots permet de savoir si <CODE>tab</CODE> contient des racines
         */
        public void set(File [] tab,boolean roots) {
            FileListModel.this.clear();
            FileListExplorer.this.nbFiles = 0;
            FileListExplorer.this.nbDirectories = 0;
            
            if (!roots) {
                if (tab != null) {
                    Vector v = new Vector();
                    for (int i=0;i<tab.length;i++) {
                        if (FileListExplorer.this.hiddenFiles) {
                            v.add(new LabelFile(tab[i],false));
                            if (tab[i].isFile())
                                FileListExplorer.this.nbFiles++;
                            else if (tab[i].isDirectory())
                                FileListExplorer.this.nbDirectories++;
                        }
                        else if(!tab[i].isHidden()) {
                            v.add(new LabelFile(tab[i],false));
                            if (tab[i].isFile())
                                FileListExplorer.this.nbFiles++;
                            else if (tab[i].isDirectory())
                                FileListExplorer.this.nbDirectories++;
                        }
                    }
                    LabelFile[] tabBis = new LabelFile[v.size()];
                    for (int i=0;i<tabBis.length;i++)
                        tabBis[i] = (LabelFile)v.elementAt(i);
                    
                    Arrays.sort(tabBis);
                    for (int i=0;i<tabBis.length;i++)
                        FileListModel.this.addElement(tabBis[i]);
                }
                FileListModel.this.insertElementAt("..",0);
                FileListModel.this.insertElementAt(".",0);
            }
            else for (int i=0;i<tab.length;i++)
                FileListModel.this.addElement(new LabelFile(tab[i],true));
        }
    }
    
    /** Classe permettant d'afficher correctement un fichier dans la liste (en renvoyant uniquement le nom
     * du fichier et pas tout son chemin d'accs).
     */
    private final class LabelFile implements Comparable,Serializable {
        /** Fichier correspondant
         */
        protected File file;
        /** Boolen permettant de savoir si un fichier est une racine ou non
         */
        boolean root;
        
        /** Constructeur crant un <CODE>LabelFile</CODE>
         * @param f fichier associ
         * @param r boolen dterminant si le fichier est une racine
         */
        public LabelFile(File f,boolean r) {
            LabelFile.this.file = f;
            LabelFile.this.root = r;
        }
        
        /** Renvoie le nom du fichier
         * @return nom du fichier
         */
        public String toString() {
            File[] tab = File.listRoots();
            for (int i=0;i<tab.length;i++) {
                if (LabelFile.this.file.equals(tab[i]))
                    return LabelFile.this.file.getPath();
            }
            return file.getName();
        }
        
        /** Fonction permettant de comparer les fichiers pour pouvoir les trier.
         * @param x objet <I>(LabelFile)</I>  comparer avec le <CODE>LabelFile</CODE> courant
         * @return entier <0, >0 ou =0
         */
        public int compareTo(final java.lang.Object x) {
            
            
            if (LabelFile.this.file.isDirectory() && !((LabelFile)x).file.isDirectory())
                return -1;
            if (!LabelFile.this.file.isDirectory() && ((LabelFile)x).file.isDirectory())
                return 1;
            
            if (FileListExplorer.this.sort == FileListExplorer.SORT_BY_NAME)
                return (LabelFile.this.file.getPath().toLowerCase().compareTo(((LabelFile)x).file.getPath().toLowerCase()));
            
            if (FileListExplorer.this.sort == FileListExplorer.SORT_BY_EXTENSION) {
                if (LabelFile.this.file.isDirectory())
                    return (LabelFile.this.file.getPath().toLowerCase().compareTo(((LabelFile)x).file.getPath().toLowerCase()));
                if (FileListExplorer.getExtension(LabelFile.this.file).toLowerCase().equals(FileListExplorer.getExtension(((LabelFile)x).file).toLowerCase()))
                    return (LabelFile.this.file.getPath().toLowerCase().compareTo(((LabelFile)x).file.getPath().toLowerCase()));
                return FileListExplorer.getExtension(LabelFile.this.file).toLowerCase().compareTo(FileListExplorer.getExtension(((LabelFile)x).file).toLowerCase());
            }
            else return 0;
        }
    }
    
    /** Classe permettant de rprsenter les lments de la liste avec des cones.
     */
    private final class LabelFileCellRenderer extends JLabel implements ListCellRenderer,Serializable {
        
        private final Border noFocusBorder;
        
        public LabelFileCellRenderer() {
            super();
            noFocusBorder = new EmptyBorder(1, 1, 1, 1);
            setOpaque(true);
            setBorder(noFocusBorder);
        }
        
        /** Renvoie un composant dont la mthode <CODE>paint</CODE> dessinera le contenu de la cellule.
         * Si les cellules de la liste n'ont pas une taille fixe, ce composant doit aussi implmenter <CODE>getPreferedSize</CODE>.
         * @param list la liste dont la cellule est en cours de dessin
         * @param value lment  dessiner
         * @param index indice de stockage de l'lment dans le modle
         * @param isSelected vrai si la cellule spcifie a t slectionne
         * @param cellHasFocus vrai si le focus d'entre est sur la cellule spcifie
         * @return composant dont la mthode <CODE>paint</CODE> dessinera le contenu de la cellule
         */
        public Component getListCellRendererComponent(
        JList list,
        Object value,
        int index,
        boolean isSelected,
        boolean cellHasFocus) {
            String s = value.toString();
            LabelFileCellRenderer.this.setText(s);
            if (s.equals("."))
                LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_REFRESH_ICON]);
            else if (s.equals(".."))
                LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_PARENT_ICON]);
            else {
                LabelFile lf = (LabelFile)value;
                if (lf.root)
                    LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_ROOT_ICON]);
                else if (!lf.file.isDirectory())
                    LabelFileCellRenderer.this.testSpecialIcons(lf.file);
                else LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_DIRECTORY_ICON]);
            }
            if (isSelected) {
                LabelFileCellRenderer.this.setBackground(list.getSelectionBackground());
                this.setForeground(list.getSelectionForeground());
            }
            else {
                LabelFileCellRenderer.this.setBackground(list.getBackground());
                LabelFileCellRenderer.this.setForeground(list.getForeground());
            }
            LabelFileCellRenderer.this.setEnabled(list.isEnabled());
            LabelFileCellRenderer.this.setFont(list.getFont());
            setBorder((cellHasFocus) ? new LineBorder(FileListExplorer.this.selectionBorderColor) : noFocusBorder);
            
            return this;
        }
        
        /** Teste si un cone est valide, si oui affecte cet cone  l'lment courant
         * @param img chemin d'accs de l'cone
         */
        private void testIcon(URL img) {
            if (img != null)
                LabelFileCellRenderer.this.setIcon(new ImageIcon(img));
            else LabelFileCellRenderer.this.setIcon(new ImageIcon(""));
        }
        
        /** Teste si un fichier  besoin d'un cone spcial contenu dans la <CODE>HashMap specialIcons</CODE>
         * @param f fichier  tester
         */
        private void testSpecialIcons(File f) {
            if ((specialIcons == null) || (specialIcons.size() < 1))
                LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_FILE_ICON]);
            else {
                URL ext = (URL)FileListExplorer.this.specialIcons.get(FileListExplorer.this.getExtension(f).toUpperCase());
                if (ext != null)
                    LabelFileCellRenderer.this.testIcon(ext);
                else LabelFileCellRenderer.this.testIcon(basicIcons[INDEX_FILE_ICON]);
            }
        }
    }
    
    /** Constructeur sans paramtres qui cr une instance d'un <CODE>FileListExplorer</CODE> en affichant les racines <CODE>(currentDirectory==null)</CODE>
     */
    public FileListExplorer() {
        init();
    }
    
    /** Constructeur crant une instance de <CODE>FileListExplorer</CODE>
     * @param f rpertoire courant (si f est un rpertoire)
     */
    public FileListExplorer(File f) {
        init();
        setCurrentDirectory(f);
    }
    
    /** Initialisation de la liste
     */
    private void init() {
        
        this.setModel(model);
        this.setCellRenderer(new LabelFileCellRenderer());
        
        this.addMouseListener(new MouseAdapter() {
            public void mouseClicked(final java.awt.event.MouseEvent evt) {
                if (evt.getClickCount() == 2)
                    changeDirectory(locationToIndex(evt.getPoint()));
            }
        });
        
        this.addKeyListener(new KeyAdapter() {
            public void keyReleased(final java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ENTER)
                    changeDirectory(getSelectedIndex());
            }
        });
        
        this.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(final javax.swing.event.ListSelectionEvent evt) {
                if (getSelectedIndex() >= 0)
                    evtSelectionFile(getSelectedIndex());
            }
        });
        model.set(File.listRoots(),true);
    }
    
    /** Mthode appelle lorsque un rpertoire ou un fichier est selectionn et qui appelle <CODE>fireFileSelected</CODE>
     * @param index index dans la liste de l'lment slctionn
     */
    private void evtSelectionFile(int index) {
        if (model.getElementAt(index).equals("."))
            this.fireFileSelected(new FileListExplorerEvent(this));
        else  if (model.getElementAt(index).equals(".."))
            this.fireFileSelected(new FileListExplorerEvent(this));
        else this.fireFileSelected(new FileListExplorerEvent(this));
    }
    
    /** Change de rpertoire courant
     * @param f nouveau rpertoire courant (si f est un rpertoire sinon la fonction n'a aucun effet)
     */
    public void setCurrentDirectory(File f) {
        if (f == null) {
            model.set(File.listRoots(),true);
            currentDirectory = null;
            this.fireDirectoryChanged(new FileListExplorerEvent(this));
            this.setSelectedIndex(0);
        }
        else if (f.isDirectory()) {
            model.set(f.listFiles(),false);
            currentDirectory = f;
            this.fireDirectoryChanged(new FileListExplorerEvent(this));
            this.setSelectedIndex(0);
        }
        else if (f.isFile()) {
            this.fireFileChoosen(new FileListExplorerEvent(this));
        }
    }
    
    /** Ractualise l'affichage de l'explorateur
     */
    public void refresh() {
        this.setCurrentDirectory(this.currentDirectory);
    }
    
    /** Renvoie le fichier ou le rpertoire slctionn
     * @return rpertoire ou fichier slctionn, <CODE>null</CODE> si aucun slctionn
     */
    public File getSelectedFile() {
        Object value = this.getSelectedValue();
        if (value == null)
            return null;
        if (value.equals(".") || value.equals(".."))
            return null;
        return ((LabelFile)value).file;
    }
    
    /** Renvoie le rpertoire courant
     * @return rpertoire courant, <CODE>null</CODE> si les racines sont affiches
     */
    public File getCurrentDirectory() {
        return currentDirectory;
    }
    
    /** Renvoie le tableau contenant les URLs des cones de base
     * @return tableau des cones de base, pour rcuprer un URL particulier il
     * suffit d'utiliser les constantes <CODE>INDEX_XXX_ICON</CODE><BR /><BR />ex:
     * <CODE>FileListExplorer fle = new FileListExplorer();<BR />
     * URL [] icons = fle.getBasicIcons();<BR />
     * URL icon = icons[INDEX_FILE_ICON];<BR />
     * </CODE>La variable <CODE>icon</CODE> contient l'URL de l'cone pour les fichiers
     */
    public URL[] getBasicIcons() {
        return basicIcons;
    }
    /** Renvoie l'URL de l'cone pour les fichiers
     * @return URL de l'cone pour les fichiers
     */
    public URL getFileIcon() {
        return basicIcons[INDEX_FILE_ICON];
    }
    
    /** Permet de changer l'URL de l'cone pour les fichiers
     * @param url nouvelle URL pour l'cone des fichiers */
    public void setFileIcon(URL url) {
        basicIcons[INDEX_FILE_ICON] = url;
    }
    /** Renvoie l'URL de l'cone pour les rpertoires
     * @return URL de l'cone pour les rpertoires
     */
    public URL getDirectoryIcon() {
        return basicIcons[INDEX_DIRECTORY_ICON];
    }
    
    /** Permet de changer l'URL de l'cone pour les rpertoires
     * @param url nouvelle URL pour l'cone des rpertoires
     */
    public void setDirectoryIcon(URL url) {
        basicIcons[INDEX_DIRECTORY_ICON] = url;
    }
    /** Renvoie l'URL de l'cone pour l'lment actualiser (.)
     * @return URL de l'cone pour l'lment actualiser
     */
    public URL getRefreshIcon() {
        return basicIcons[INDEX_REFRESH_ICON];
    }
    
    /** Permet de changer l'URL de l'cone pour l'lment actualiser (.)
     * @param url nouvelle URL pour l'cone de l'lment actualiser
     */
    public void setRefreshIcon(URL url) {
        basicIcons[INDEX_REFRESH_ICON] = url;
    }
    
    /** Renvoie l'URL de l'cone pour l'lment rpertoire parent (..)
     * @return URL de l'cone pour l'lment rpertoire parent
     */
    public URL getParentIcon() {
        return basicIcons[INDEX_PARENT_ICON];
    }
    
    /** Permet de changer l'URL de l'cone pour l'lment rpertoire parent (..)
     * @param url nouvelle URL pour l'cone de l'lment rpertoire parent
     */
    public void setParentIcon(URL url) {
        basicIcons[INDEX_PARENT_ICON] = url;
    }
    
    /** Renvoie l'URL de l'cone pour les racines (C:,D:,/,...)
     * @return URL de l'cone pour les racines
     */
    public URL getRootIcon() {
        return basicIcons[INDEX_ROOT_ICON];
    }
    
    /** Permet de changer l'URL de l'cone pour les racines (C:,D:,/,...)
     * @param url nouvelle URL pour l'cone des racines
     */
    public void setRootIcon(URL url) {
        basicIcons[INDEX_ROOT_ICON] = url;
    }
    
    /** Renvoie une table <I>(HashMap)</I> contenant les URLs des cones pour des fichiers spciaux avec comme cles,
     * l'extension des fichiers <I>(String)</I> en majuscules.
     * @return <CODE>HashMap</CODE> de la forme suivante : <BR /><BR />
     * <TABLE BORDER='1'>
     * <TR>
     * <TD>Cles</TD><TD>Chemins des cones</TD>
     * </TR>
     * <TR>
     * <TD>".JAVA"</TD><TD>URL "imgjava.jpg"</TD>
     * </TR>
     * <TR>
     * <TD>".CLASS"</TD><TD>URL "imgclass.jpg"</TD>
     * </TR>
     * <TR>
     * <TD>".TXT"</TD><TD>URL "txt.gif"</TD>
     * </TR>
     * </TABLE>
     * <BR />Renvoie <CODE>null</CODE> si il n'y a pas d'cones spciaux
     */
    public HashMap getSpecialIcons() {
        return specialIcons;
    }
    
    /** Permet de changer la table contenant les extensions et les URLs des cones des fichiers spciaux
     * @param t <CODE>HashMap</CODE> contenant comme cles les extensions des fichiers et comme valeurs les URLs des cones
     * <BR /><BR />ex:
     * <CODE>
     * FileListExplorer fle = new FileListExplorer();<BR />
     * HashMap hm = new HashMap();<BR />
     * hm.put(".TXT",this.getClass().getRessource("txt.gif"));<BR />
     * hm.put(".JAVA",this.getClass().getRessource("java.gif"));<BR />
     * fle.setSpecialIcons(hm);<BR />
     * </CODE>
     * Les fichiers .txt et .java seront donc affichs avec des cones spciaux
     */
    public void setSpecialIcons(HashMap t) {
        specialIcons = t;
    }
    
    /** Renvoie le nombre de fichiers dans le rpertoire courant
     * @return nombre de fichiers
     */
    public int getNbFiles() {
        return this.nbFiles;
    }
    
    /** Renvoie le nombre de sous-rpertoires du rpertoire courant
     * @return nombre de sous-rpertoires
     */
    public int getNbDirectories() {
        return this.nbDirectories;
    }
    
    /** Renvoie l'extension d'un fichier
     * @param f fichier dont il faut rcuprer l'extension
     * @return l'extension du fichier prcde du "." (ex: ".java")
     */
    public final static String getExtension(File f) {
        String path = f.getName();
        int i = path.lastIndexOf('.');
        if (i<=0)
            return "";
        return path.substring(i);
    }
    
    /** Change de rpertoire courant
     * @param index index de l'lment slctionn dans la liste
     */
    private final void changeDirectory(int index) {
        if (index > -1) {
            if (model.getElementAt(index).equals("..")) {
                this.setCurrentDirectory(currentDirectory.getParentFile());
            }
            else if (model.getElementAt(index).equals(".")) {
                this.setCurrentDirectory(currentDirectory);
            }
            else this.setCurrentDirectory(((LabelFile)model.getElementAt(index)).file);
        }
    }
    
    /** Cration d'un nouveau rpertoire dans le rpertoire courant
     * @param name nom du nouveau rpertoire
     */
    public final void mkDir(String name) {
        if (currentDirectory != null) {
            File newDir = new File(currentDirectory.getPath()+ FILE_SEPARATOR + name);
            if (newDir.mkdir())
                this.setCurrentDirectory(currentDirectory);
        }
    }
    
    /** Suppression du rperoire (si vide) ou du fichier slectionn
     */
    public final void delete() {
        File f = this.getSelectedFile();
        if (f != null) {
            if (f.delete()) {
                f.delete();
                this.setCurrentDirectory(currentDirectory);
            }
        }
    }
    
    /** Retourne la couleur de la bordure de la slction
     * @return couleur de la bordure de la slction
     */
    public Color getSelectionBorderColor() {
        return this.selectionBorderColor;
    }
    
    /** Change la couleur de la bordure de la slction
     * @param c nouvelle couleur
     */
    public void setSelectionBorderColor(Color c) {
        this.selectionBorderColor = c;
    }
    
    /** Teste si les fichiers et dossiers cachs sont affichs
     * @return vrai si les fichiers cachs sont affichs, faux sinon
     */
    public boolean isShowHiddenFiles() {
        return this.hiddenFiles;
    }
    
    /** Permet l'affichage ou non des fichiers cachs
     * @param vrai pour afficher les fichiers cachs, faux sinon
     */
    public void setShowHiddenFiles(boolean b) {
        this.hiddenFiles = b;
    }
    
    /** Retourne le type de tri des fichiers
     * @return type de tri (voir <CODE>SORT_BY_NAME</CODE> et <CODE>SORT_BY_EXTENSION</CODE>)
     */
    public int getSortType() {
        return this.sort;
    }
    
    /** Change le type de tri des fichiers
     * @param s nouveau type de tri (voir <CODE>SORT_BY_NAME</CODE> et <CODE>SORT_BY_EXTENSION</CODE>)
     */
    public void setSortType(int s) {
        this.sort = s;
    }
    
    /** Liste des <I>listeners</I> enregistrs pour ce bean
     */
    transient protected Vector m_listeners = new Vector();
    
    /** Ajoute un nouveau couteur <I>(listener)</I> dans la liste des couteurs du bean
     * @param l nouveau couteur <I>(listener)</I>
     */
    public void addFileListExplorerListener(FileListExplorerListener l) {
        synchronized(m_listeners){
            m_listeners.addElement(l);
        }
    }
    
    /** Supprime un couteur <I>(listener)</I> de la liste des couteurs du bean
     * @param l couteur  supprimer
     */
    public void removeFileListExplorerListener(FileListExplorerListener l) {
        synchronized(m_listeners){
            m_listeners.remove(l);
        }
    }
    
    /** Notification de l'vnement <CODE>fileSelected</CODE> aux couteurs
     * @param evt objet vnement associ
     */
    protected void fireFileSelected(FileListExplorerEvent evt) {
        Vector listeners;
        synchronized(m_listeners){
            listeners = (Vector)m_listeners.clone();
        }
        for (int i=0, n=listeners.size();i<n;i++) {
            FileListExplorerListener l = (FileListExplorerListener)listeners.elementAt(i);
            l.fileSelected(evt);
        }
    }
    
    /** Notification de l'vnement <CODE>fileChoosen</CODE> aux couteurs
     * @param evt objet vnement associ
     */
    protected void fireFileChoosen(FileListExplorerEvent evt) {
        Vector listeners;
        synchronized(m_listeners){
            listeners = (Vector)m_listeners.clone();
        }
        for (int i=0, n=listeners.size();i<n;i++) {
            FileListExplorerListener l = (FileListExplorerListener)listeners.elementAt(i);
            l.fileChoosen(evt);
        }
    }
    
    /** Notification de l'vnement <CODE>directoryChanged</CODE> aux couteurs
     * @param evt objet vnement associ
     */
    protected void fireDirectoryChanged(FileListExplorerEvent evt) {
        Vector listeners;
        synchronized(m_listeners){
            listeners = (Vector)m_listeners.clone();
        }
        for (int i=0, n=listeners.size();i<n;i++) {
            FileListExplorerListener l = (FileListExplorerListener)listeners.elementAt(i);
            l.directoryChanged(evt);
        }
    }
    
    /** Table <I>(HashMap)</I> contenant les URLs des cones spciaux avec comme cls les extensions des fichiers spciaux en majuscules (ex: .JAVA)
     */
    private HashMap specialIcons;
    /** Tableau contenant les URLs des cones de base <I>(file, directory, refresh, parent, root)</I>
     */
    private URL [] basicIcons= {DEFAULT_FILE_ICON,DEFAULT_DIRECTORY_ICON,DEFAULT_REFRESH_ICON,DEFAULT_PARENT_ICON,DEFAULT_ROOT_ICON};
    /** Nombre de fichiers dans le rpertoire courant
     */
    private int nbFiles = 0;
    /** Nombre de sous-rpertoires dans le rpertoire courant
     */
    private int nbDirectories = 0;
    /** Objet contenant les lments de la liste
     */
    private FileListModel model = new FileListModel();
    /** Rpertoire courant
     */
    private File currentDirectory = null;
    /** Couleur de la bordure de la slction
     */
    private Color selectionBorderColor = null;
    /** Affichage des fichiers cachs
     */
    private boolean hiddenFiles = true;
    /** Type de tri des fichiers et rpertoires
     */
    private int sort = SORT_BY_NAME;
}
