/*
 * 09/04/2002
 *
 * Elife
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.plaf.metal.MetalLookAndFeel;
import tvouriot.elife.gui.*;
import tvouriot.elife.menus.*;
import tvouriot.elife.actions.*;
import tvouriot.elife.images.*;
import org.gjt.sp.jedit.textarea.DefaultInputHandler;

/** Class principale
 */
public class Elife {
    
    /** Version */
    public static final String RELEASE = "1.1.5";
    
    /** Build */
    public static final String BUILD = "020624";
    
    /** Site web */
    public static final String WEB = "http://thierry.vouriot.free.fr";
    
    /** Mode "debug" */
    public static final boolean DEBUG = false;
    
    /** Rpertoire home */
    public static final File HOME_DIRECTORY = new File(System.getProperty("user.home"));
    
    /** Proprits de l'application */
    private static Properties props = new Properties();
    
    /** Fichier permettant la sauvegarde des proprits */
    private static final String FILE_PROPERTIES = ".elife.properties";
    
    /** Chemin d'accs du fichier FILE_PROPERTIES */
    private static final String USER_PROPS = HOME_DIRECTORY + File.separator + FILE_PROPERTIES;
    
    /** Table contenant toutes les actions */
    private static final HashMap actionHash = new HashMap();
    
    /** Nombre de fichiers "rcents"  sauvegarder dans les proprits */
    private static final int NB_RECENTS = 10;
    
    /** Chemin d'accs du rpertoire o est install Elife */
    private static String elifeHome = null;
    
    /** Ajouter une action  la table
     * @param action action listener
     */
    public static void addAction(MenuAction action) {
        actionHash.put(action.getName(), action);
    }
    
    /** Retourne une action  partir de son nom
     * @param action nom de l'action
     * @return l'action
     */
    public static MenuAction getAction(String action) {
        return (MenuAction) actionHash.get(action);
    }
    
    /** Initialise la table des actions en y ajoutant toutes les actions possibles
     */
    public static void initActions() {
        
        addAction(new ActionAbout());
        addAction(new ActionClose());
        addAction(new ActionCloseAll());
        addAction(new ActionExit());
        addAction(new ActionTools());
        addAction(new ActionNew());
        addAction(new ActionOpen());
        addAction(new ActionSave());
        addAction(new ActionSaveAs());
        addAction(new ActionSaveAll());
        addAction(new ActionSelectAll());
        addAction(new ActionUndo());
        addAction(new ActionRedo());
        addAction(new ActionCut());
        addAction(new ActionPaste());
        addAction(new ActionCopy());
        addAction(new ActionFind());
        addAction(new ActionFindNext());
        addAction(new ActionReplace());
        addAction(new ActionGoto());
        addAction(new ActionRecent());
        addAction(new ActionClearRecents());
        addAction(new ActionOptions());
        addAction(new ActionSyntax());
        addAction(new ActionToolbar());
        addAction(new ActionStatusbar());
        addAction(new ActionDoc());
    }
    
    /** Initialise les proprits en les chargeant  partir du fichier de sauvegarde
     */
    public static void initProperties() {
        
        if (new File(USER_PROPS).exists()) {
            try {
                loadProps(new FileInputStream(USER_PROPS));
            } catch (FileNotFoundException fnfe) { }
            //Teste si le fichier de proprits correspond  la bonne version de Elife
            if (getProperty("release") == null || !getProperty("release").equals(RELEASE.substring(0,3))) {
                loadProps(Elife.class.getResourceAsStream(FILE_PROPERTIES));
                setProperty("explorerdir",HOME_DIRECTORY.getPath());
            }
        }
        else {
            loadProps(Elife.class.getResourceAsStream(FILE_PROPERTIES));
            setProperty("explorerdir",HOME_DIRECTORY.getPath());
        }
    }
    
    /** Sauvegarde les proprits dans un fichier sur le disque dur
     */
    public static void saveProps() {
        try {
            OutputStream out = new FileOutputStream(USER_PROPS);
            props.store(out, "Elife Properties");
            out.close();
        } catch (IOException io) { }
    }
    
    /** Charge un fichier contenant les proprits de l'application
     * @param in un <code>InputStream</code> correspondant au fichier
     */
    public static void loadProps(InputStream in) {
        try {
            props.load(new BufferedInputStream(in));
            in.close();
        } catch (IOException ioe) { }
    }
    
    /** Changer une proprit
     * @param name nom de la proprit
     * @param value la nouvelle valeur
     */
    public static void setProperty(String name, String value) {
        props.put(name, value);
    }
    
    /** Supprime une proprit
     * @param name nom de la proprit  supprimer
     */
    public static void removeProperty(String name) {
        props.remove(name);
    }
    
    /** Renvoie la valeur d'une proprit
     * @param name nom de la proprit  lire
     * @return la valeur de la proprit spcifie
     */
    public static String getProperty(String name) {
        return props.getProperty(name);
    }
    
    /** Ajoute dans les proprits le chemin d'accs d'un fichier "rcent"
     * @param f fichier "rcent"  ajouter
     */
    public static void addRecent(File f) {
        boolean exist = false;
        String path = f.getPath();
        int i;
        for (i=0;!exist && i<NB_RECENTS;i++) {
            String r = getProperty("recent."+i);
            if (r == null)
                break;
            else if (path.equals(r))
                exist = true;
        }
        if (!exist) {
            
            for (i=NB_RECENTS-1; i>0; i--) {
                int k = i - 1;
                String r = getProperty("recent."+k);
                if (r != null)
                    setProperty("recent."+i,r);
            }
            setProperty("recent.0",path);
        }
        path = null;
        ElifeMenuBar.loadRecents();
    }
    
    /** Efface des proprits tous les fichiers rcents
     */
    public static void clearRecents() {
        
        for (int i=0;i<NB_RECENTS;i++) {
            if (getProperty("recent."+i) == null)
                break;
            else removeProperty("recent."+i);
        }
        ElifeMenuBar.loadRecents();
    }
    
    /** Teste la valeur de la proprits <CODE>flatmenus</CODE>
     * @return vrai pour valeur "on", faux pour valeur "off"
     */
    public static boolean isFlatMenus() {
        return Elife.getProperty("flatmenus").equals("on");
    }
    
    /** Change la valeur de la proprits <CODE>flatmenus</CODE>
     * @param bool vrai pour valeur "on", faux pour valeur "off"
     */
    public static void setFlatMenus(boolean bool) {
        Elife.setProperty("flatmenus",bool ? "on" : "off");
    }
    
    /** Initialise le look & feel Metal
     */
    public static void initUI() {
        MetalLookAndFeel.setCurrentTheme(new ElifeMetalTheme());
    }
    
    /** Retourne le chemin d'accs du rpertoire o est install Elife
     *  @return chemin d'accs d'Elife
     */
    public static String getElifeHome() {
        
        if (elifeHome == null) {
            String classpath = System.getProperty("java.class.path");
            int index = classpath.toLowerCase().indexOf("elife.jar");
            int start = classpath.lastIndexOf(File.pathSeparator,index) + 1;
            // if started with java -jar Elife.jar
            if(classpath.equalsIgnoreCase("elife.jar")) {
                elifeHome = System.getProperty("user.dir");
            }
            else if(index > start) {
                elifeHome = classpath.substring(start,index - 1);
            }
            else elifeHome = System.getProperty("user.dir");
        }
        return elifeHome;
    }
    
    /** Point de dpart de l'application
     * @param argv arguments de la ligne de commande
     */
    public static void main(String [] argv) {
        SplashScreen splash = new SplashScreen(Utilities.getURL(Icons.IMGABOUT,Elife.class));
        try {
            java.lang.Thread.sleep(1100);
            
        } catch(Exception e) { }
        ElifeFrame frm = new ElifeFrame();
        frm.show();
        splash.setTerminated();
        
        frm.getTabbedPane().requestFocusForText(0);
        frm.validate();
    }
}