/*
 * 23/04/2002
 *
 * DocBrowser
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import tvouriot.elife.*;
import tvouriot.elife.images.*;

/** Class permettant l'affichage de la documentation */
public class DocBrowser extends JFrame {
    
    /** Champs de texte contenant l'url du document affich */
    private JTextField txtUrl = new JTextField(50);
    /** Bouton "Prcdent" */
    private ElifeButton cmdPrev = new ElifeButton(Utilities.getIcon(Icons.BACK, Elife.class),"Prcdent");
    /** Bouton "Suivant" */
    private ElifeButton cmdNext = new ElifeButton(Utilities.getIcon(Icons.GOTO, Elife.class),"Suivant");
    /** Browser HTML permettant l'affichage de la doc */
    private HTMLViewer viewer = new HTMLViewer();
    /** Arbre reprsentant le sommaire de la doc */
    private ContentsTree contentsTree = new ContentsTree();
    
    /* URLs des fichiers de la documentation */
    static final String INSTALL = "/doc/install.htm";
    static final String HISTORY = "/doc/history.htm";
    static final String ABOUT = "/doc/about.htm";
    static final String GNU = "/doc/gnu.htm";
    static final String MENUS = "/doc/menus.htm";
    static final String MNU_FILE = "/doc/mnu_file.htm";
    static final String MNU_EDIT = "/doc/mnu_edit.htm";
    static final String MNU_SEARCH = "/doc/mnu_search.htm";
    static final String MNU_ELIFE = "/doc/mnu_elife.htm";
    static final String OPTIONS = "/doc/options.htm";
    static final String EXPLORER= "/doc/explorer.htm";
    static final String EDITOR = "/doc/editor.htm";
    
    /** Constructeur */
    public DocBrowser() {
        setTitle("Elife Documentation");
        setSize(640, 500);
        setLocation(50,50);
        setIconImage(this.getToolkit().createImage(Utilities.getURL(Icons.ELIFE,Elife.class)));
        
        JScrollPane scrollContents = new JScrollPane(contentsTree);
        JPanel pnlContents = new JPanel(new BorderLayout());
        pnlContents.add(scrollContents,BorderLayout.CENTER);
        pnlContents.setPreferredSize(new Dimension(185,200));
        pnlContents.setMinimumSize(new Dimension(185,200));
        JScrollPane scrollViewer = new JScrollPane(viewer);
        
        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,pnlContents,scrollViewer);
        split.setOneTouchExpandable(true);
        split.setDividerSize(10);
        
        JToolBar toolBar = new JToolBar();
        toolBar.add(cmdPrev);
        toolBar.add(cmdNext);
        toolBar.addSeparator();
        toolBar.add(new JLabel("  Location :  "));
        toolBar.add(txtUrl);
        toolBar.setFloatable(false);
        
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout(5,5));
        contentPane.add(split, BorderLayout.CENTER);
        contentPane.add(toolBar, BorderLayout.NORTH);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        } );
        
        cmdPrev.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                viewer.prevPage();
            }
        });
        
        cmdNext.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                viewer.nextPage();
            }
        });
        
        try {
            viewer.setPage(new File(Elife.getElifeHome() + ABOUT).toURL());
        }
        catch(Exception e) { viewer.setText("Documentation non disponible:\n"+e.toString()); }
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        contentsTree = null;
        viewer = null;
        txtUrl = null;
        cmdPrev = null;
        cmdNext = null;
        super.finalize();
    }
    
    /** Zone de texte permettant l'affichage des fichiers HTML et la gestion des
     *  pages prcdentes et suivantes
     */
    private final class HTMLViewer extends JEditorPane {
        
        private Stack prevStack = new Stack();
        private Stack nextStack = new Stack();
        
        public HTMLViewer() {
            this.setEditable(false);
            this.setFont(new Font("Arial",Font.PLAIN,12));
        }
        
        public void setPage(URL url) {
            if (url!=null)
                try {
                    super.setPage(url);
                    prevStack.push(url);
                    txtUrl.setText(url.toString());
                }
                catch(IOException e) { this.setText("Documentation non disponible:\n"+e.toString()); }
        }
        
        public void prevPage() {
            if (prevStack.size() > 1) {
                nextStack.push((URL)prevStack.pop());
                setPage((URL)prevStack.peek());
                prevStack.pop();
            }
            
        }
        
        public void nextPage() {
            if (nextStack.size() > 0) {
                setPage((URL)nextStack.pop());
            }
        }
    }
    
    /** Arbre reprsentant le sommaire de la doc */
    private final class ContentsTree extends JTree implements TreeSelectionListener {
        
        private final class ContentsURL {
            public String name;
            public URL url = null;
            
            public ContentsURL(String n,String path) {
                name = n;
                if (path != null)
                    try {
                        url = new File(Elife.getElifeHome() + path).toURL();
                    }
                    catch(Exception e) { path = null; }
            }
            
            public String toString() { return name; }
        }
        
        public ContentsTree() {
            this.setFont(new Font("Dialog",Font.PLAIN,12));
            DefaultMutableTreeNode ndeRoot = new DefaultMutableTreeNode("Sommaire");
            DefaultMutableTreeNode ndeInstall = new DefaultMutableTreeNode(new ContentsURL("Installer Elife",INSTALL));
            DefaultMutableTreeNode ndeHistory = new DefaultMutableTreeNode(new ContentsURL("Historique des versions",HISTORY));
            DefaultMutableTreeNode ndeAbout = new DefaultMutableTreeNode(new ContentsURL("A propos de Elife",ABOUT));
            DefaultMutableTreeNode ndeGnu = new DefaultMutableTreeNode(new ContentsURL("Licence GNU",GNU));
            DefaultMutableTreeNode ndeUserGuide = new DefaultMutableTreeNode(new ContentsURL("Guide utilisateur",null));
            DefaultMutableTreeNode ndeMenus = new DefaultMutableTreeNode(new ContentsURL("Menus",null));
            DefaultMutableTreeNode ndeFichier = new DefaultMutableTreeNode(new ContentsURL("Fichier",MNU_FILE));
            DefaultMutableTreeNode ndeEdition = new DefaultMutableTreeNode(new ContentsURL("Edition",MNU_EDIT));
            DefaultMutableTreeNode ndeRecherche = new DefaultMutableTreeNode(new ContentsURL("Recherche",MNU_SEARCH));
            DefaultMutableTreeNode ndeElife = new DefaultMutableTreeNode(new ContentsURL("Elife",MNU_ELIFE));
            DefaultMutableTreeNode ndeEditeur = new DefaultMutableTreeNode(new ContentsURL("Editeur",EDITOR));
            DefaultMutableTreeNode ndeExplorateur = new DefaultMutableTreeNode(new ContentsURL("Explorateur",EXPLORER));
            DefaultMutableTreeNode ndeOptions = new DefaultMutableTreeNode(new ContentsURL("Options",OPTIONS));
            ndeRoot.add(ndeAbout);
            ndeRoot.add(ndeInstall);
            ndeRoot.add(ndeHistory);
            ndeRoot.add(ndeGnu);
            ndeRoot.add(ndeUserGuide);
            ndeUserGuide.add(ndeMenus);
            ndeUserGuide.add(ndeEditeur);
            ndeUserGuide.add(ndeExplorateur);
            ndeUserGuide.add(ndeOptions);
            ndeMenus.add(ndeFichier);
            ndeMenus.add(ndeEdition);
            ndeMenus.add(ndeRecherche);
            ndeMenus.add(ndeElife);
            this.setModel(new DefaultTreeModel(ndeRoot));
            this.setRootVisible(false);
            this.expandRow(4);
            this.expandRow(5);
            this.setShowsRootHandles(true);
            this.putClientProperty("JTree.lineStyle","Angled");
            this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            this.setPreferredSize(new Dimension(180,180));
            this.setMinimumSize(new Dimension(180,180));
            this.addTreeSelectionListener(this);
        }
        
        public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
            TreePath path = this.getSelectionPath();
            if (path == null) return;
            DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)path.getLastPathComponent();
            ContentsURL c = (ContentsURL)selectedNode.getUserObject();
            viewer.setPage(c.url);
        }
    }
}

