/*
 * 11/01/2002
 *
 * ElifeButton
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** Class dfinissant les boutons d'Elife
 */
public class ElifeButton extends JButton {
    
    /** Boolen dfinissant si on utilise le rollover (dans ce cas lorsque la souris passe sur le bouton
     * on affiche son contour sinon on efface son contour)
     */    
    private boolean rollover = true;
   
    /** Change l'attribut rollover (dans ce cas lorsque la souris passe sur le bouton
     * on affiche son contour sinon on efface son contour)
     * @param enabled nouvelle valeur
     */    
    public void setRollover(boolean enabled) {
        rollover = enabled;
        setBorderPainted(!enabled);
    }
    
    /** Dsactive le bouton dans le cas ou le rollover est activ
     */    
    public void setOff() {
        if (rollover)
            this.setBorderPainted(false);
        this.setEnabled(false);
    }
    
    /** Initialise le bouton
     */    
    private void init() {
        if (rollover) {
            setBorderPainted(false);
            addMouseListener(new MouseHandler());
        } 
    }
    
    /** Constructeur
     */    
    public ElifeButton() {
        super();
        init();
    }
    
    /** Constructeur
     * @param icon icne du bouton
     */    
    public ElifeButton(Icon icon) {
        super(icon);
        init();
    }
    
    /** Constructeur
     * @param icon icne du bouton
     * @param toolTipText texte affich lorsque la souris reste sur le bouton
     */    
    public ElifeButton(Icon icon, String toolTipText) {
        super(icon);
        this.setMargin(new Insets(0,0,0,0));
        this.setToolTipText(toolTipText);
        init();
    }
    
    /** Constructeur
     * @param icon icne du bouton
     * @param toolTipText texte affich lorsque la souris reste sur le bouton
     * @param roll active ou non le rollover
     */    
    public ElifeButton(Icon icon, String toolTipText, boolean rool) {
        super(icon);
        this.setMargin(new Insets(0,0,0,0));
        this.setToolTipText(toolTipText);
        rollover = false;
        init();
    }
    
    /** Constructeur
     * @param text texte du bouton
     */    
    public ElifeButton(String text) {
        super(text);
        init();
    }
    
    /** Constructeur
     * @param text texte du bouton
     * @param icon icne du bouton
     */    
    public ElifeButton(String text, Icon icon) {
        super(text, icon);
        init();
    }
    
    /** Constructeur
     * @param text texte du bouton
     * @param roll active ou non le rollover
     */    
    public ElifeButton(String text, boolean roll) {
        super(text);
        rollover = false;
        init();
    }
    
    /** Class permettant la gestion de la souris
     */    
    class MouseHandler extends MouseAdapter {
        /** Fonction active lorsque la souris passe sur le bouton
         * @param me vnement souris
         */        
        public void mouseEntered(MouseEvent me) {
            if (isEnabled()) {
                if (rollover)
                    setBorderPainted(true);
            }
        }
        
        /** Fonction active lorsque la souris sort sur le bouton
         * @param me vnement souris
         */        
        public void mouseExited(MouseEvent me) {
            if (isEnabled()) {
                if (rollover)
                    setBorderPainted(false);
            }
        }
    }
}

