/*
 * 09/04/2002
 *
 * ElifeTextArea
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.gui;

import java.io.*;
import java.awt.*;
import javax.swing.text.Segment;
import javax.swing.text.Element;
import javax.swing.text.BadLocationException;
import javax.swing.event.*;
import javax.swing.undo.*;
import tvouriot.elife.*;
import tvouriot.elife.textarea.*;
import org.gjt.sp.jedit.syntax.*;
import org.gjt.sp.jedit.textarea.*;
import gnu.regexp.*;

/** Zone de texte permettant l'dition des fichiers et contenant les fonctions
 * d'ouverture, d'enregistrement et de recherche/remplacement
 */
public class ElifeTextArea extends JEditTextArea implements DocumentListener, UndoableEditListener, CaretListener {
    
    /** Variable utilise dans les fonctions de recherche et de remplacement */
    private RE regexp;
    /** Fichier actuellement ouvert */
    private File currentFile;
    /** Variable permettant de savoir si la zone de texte a t modifi sans tre enregistr
     */
    private boolean dirty;
    /** Variable permettant de savoir si le fichier est nouveau */
    private boolean newf;
    /** Objet permettant d'annuler ou de rtablir une action
     */
    private UndoManager undo = new UndoManager();
    /** Type de coloration syntaxique */
    private String syntax = "Texte";
    /** Mode auto-indentation */
    private boolean autoIndent = false;
    /** Dernier mot ayant t recherch */
    private String findPattern;
    /** Dernier mot ayant servi pour le remplacement */
    private String replacePattern;
    /** Cette constante dfinie la taille du buffer utilis pour la lecture d'un fichier */
    private static final int BUFFER_SIZE = 32768;
    
    /** Constructeur
     */
    public ElifeTextArea() {
        newf = true;
        dirty = false;
        this.addCaretListener(this);
        undo.setLimit(500);
        this.repaint();
    }
    
    /** Affiche ou cache les marqueurs de fin de lignes */
    public void changeEOLMarkersPainted() {
        painter.setEOLMarkersPainted(!painter.getEOLMarkersPainted());
        this.requestFocus();
    }
    
    /** Affiche ou cache la goutire */
    public void changeGutterPainted() {
        gutter.toggleCollapsed();
        this.requestFocus();
    }
    
    /** Active ou dsactive le mode auto-indentation */
    public void changeAutoIndentMode() {
        if (autoIndent=!autoIndent)
            inputHandler.addKeyBinding("ENTER", new IndentOnEnter());
        else {
            inputHandler.removeAllKeyBindings();
            inputHandler.addDefaultKeyBindings();
        }
        this.requestFocus();
    }
    
    /** Changer le document associ  la zone de texte
     * @param document nouveau document
     */
    public void setDocument(org.gjt.sp.jedit.syntax.SyntaxDocument document) {
        document.removeUndoableEditListener(this);
        document.removeDocumentListener(this);
        super.setDocument(document);
        document.addDocumentListener(this);
        document.addUndoableEditListener(this);
    }
    
    /** Modifier la valeur du dernier mot ayant t recherch
     * @param pattern nouveau mot
     */
    public void setFindPattern(String pattern) {
        findPattern = pattern;
    }
    
    /** Renvoie le dernier mot ayant t recherch
     * @return dernier mot ayant t recherch
     */
    public String getFindPattern() {
        return findPattern;
    }
    
    /** Modifier la valeur du dernier mot ayant servi pour le remplacement
     * @param pattern nouveau mot
     */
    public void setReplacePattern(String pattern) {
        replacePattern = pattern;
    }
    
    /** Renvoie la valeur du dernier mot ayant servi pour le remplacement
     * @return dernier mot ayant servi pour le remplacement
     */
    public String getReplacePattern() {
        return replacePattern;
    }
    
    /** Renvoie le panel contenant cette zone de texte
     * @return panel contenant cette zone de texte
     */
    private PanelTextEditor getParentPanel() {
        Container c = this.getParent();
        return (PanelTextEditor)c;
    }
    
    /** Retourne les parenthses ouvrantes d'un bloc de code en fonction
     *  du mode de coloration syntaxique
     *  @return parenthses ouvrantes
     */
    public String getOpenBrackets() {
        String res = "";
        if (syntax.equals("Texte"))
            res = "";
        else if (syntax.equals("Makefile"))
            res = "";
        else if (syntax.equals("SQL"))
            res = "";
        else if (syntax.equals("Makefile"))
            res = "";
        else if (syntax.equals("Specif"))
            res = "";
        else res = "{";
        return res;
    }
    
    /** Enregistre le contenu de la zone de texte dans fichier
     * @param file fichier dans lequel l'enregistrement sera fait
     * @return vrai si l'enregistrement a russi, faux sinon
     */
    public boolean save(File file) {
        boolean res = true;
        try {
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)), BUFFER_SIZE);
            Segment lineSegment = new Segment();
            String newline = System.getProperty("line.separator");
            Element map = getDocument().getDefaultRootElement();
            // we zip the text line by line
            for (int i = 0; i < map.getElementCount(); i++) {
                Element line = map.getElement(i);
                int start = line.getStartOffset();
                getDocument().getText(start, line.getEndOffset() - start - 1, lineSegment);
                out.write(lineSegment.array, lineSegment.offset, lineSegment.count);
                out.write(newline);
            }
            out.close();
            if (!file.equals(currentFile)) {
                currentFile = file;
            }
            if (isNew()) newf = false;
        } catch(IOException ioe) {
            Utilities.showError("Erreur lors de l'enregistrement du fichier "+file.getName()+"\n"+ioe);
            res = false;
        } catch(BadLocationException ble) {
            Utilities.showError("Erreur lors de l'enregistrement du fichier "+file.getName()+"\n"+ble);
            res = false;
        }
        return res;
    }
    
    /** Fonction tire de l'diteur Jext (http://www.jext.org)
     * Called to load a new file in the text area.
     * Determines which line separator (\n, \r\n...) are used in the
     * file to open. Convert'em into Swing line separator (\n).
     * Slava Pestov. Minor improvements by Romain Guy.
     * @param toLoad fichier  ouvrir
     */
    public void open(File toLoad) throws BadLocationException, FileNotFoundException, IOException {
        setDirty(false);
        StringBuffer buffer;
        InputStreamReader in;
        this.getDocument().removeDocumentListener(this);
        this.getDocument().removeUndoableEditListener(this);
        
        // we check if the file is read only or not
        if (!toLoad.canWrite())
            setEditable(false);
        else if (!isEditable())
            setEditable(true);
        buffer = new StringBuffer((int) toLoad.length());
        in = new InputStreamReader(new FileInputStream(toLoad));
        
        char[] buf = new char[BUFFER_SIZE];
        int len;
        int lineCount = 0;
        boolean CRLF = false;
        boolean CROnly = false;
        boolean lastWasCR = false;
        
        // we read the file till its end (amazing, hu ?)
        while ((len = in.read(buf, 0, buf.length)) != -1) {
            int lastLine = 0;
            for (int i = 0; i < len; i++) {
                switch(buf[i]) {
                    // and we convert system's carriage return char into \n
                    case '\r':
                        if (lastWasCR) {
                            CROnly = true;
                            CRLF = false;
                        } else
                            lastWasCR = true;
                        buffer.append(buf, lastLine, i - lastLine);
                        buffer.append('\n');
                        lastLine = i + 1;
                        break;
                    case '\n':
                        if (lastWasCR) {
                            CROnly = false;
                            CRLF = true;
                            lastWasCR = false;
                            lastLine = i + 1;
                        } else {
                            CROnly = false;
                            CRLF = false;
                            buffer.append(buf, lastLine, i - lastLine);
                            buffer.append('\n');
                            lastLine = i + 1;
                        }
                        break;
                    default:
                        if (lastWasCR) {
                            CROnly = true;
                            CRLF = false;
                            lastWasCR = false;
                        }
                        break;
                }
            }
            buffer.append(buf, lastLine, len - lastLine);
        }
        in.close();
        
        if (buffer.length() != 0 && buffer.charAt(buffer.length() - 1) == '\n')
            buffer.setLength(buffer.length() - 1);
        
        // we clear the area
        getDocument().remove(0, getLength());
        // we put the text in it
        getDocument().insertString(0, buffer.toString(), null);
        setCaretPosition(0);
        // we add the file into the recent menu
        
        currentFile = toLoad;
        newf = false;
        
        this.getDocument().addUndoableEditListener(this);
        this.getDocument().addDocumentListener(this);
    }
    
    /** Change la coloration syntaxique qui sera utilise
     * @param s type de coloration
     */
    public void changeSyntax(String s) {
        syntax = s;
        if (!s.equals("Texte")) {
            if (s.equals("Java"))
                this.setTokenMarker(new JavaTokenMarker());
            else if (s.equals("C"))
                this.setTokenMarker(new CTokenMarker());
            else if (s.equals("C++"))
                this.setTokenMarker(new CCTokenMarker());
            else if (s.equals("HTML/XML"))
                this.setTokenMarker(new HTMLTokenMarker());
            else if (s.equals("SQL"))
                this.setTokenMarker(new SQLTokenMarker());
            else if (s.equals("Makefile"))
                this.setTokenMarker(new MakefileTokenMarker());
            else if (s.equals("Specif"))
                this.setTokenMarker(new SpecTokenMarker());
            else if (s.equals("PHP"))
                this.setTokenMarker(new PHPTokenMarker());
            else if (s.equals("Python"))
                this.setTokenMarker(new PythonTokenMarker());
        }
        else this.setTokenMarker(null);
        this.repaint();
        this.requestFocus();
    }
    
    /** Initialise la coloration syntaxique qui sera utilise en
     * fonction de l'extension du fichier
     */
    public void initSyntax() {
        String ext = Utilities.getFileExtension(currentFile).toLowerCase();
        if (ext.equals(".java") || ext.equals(".jav")) {
            this.setTokenMarker(new JavaTokenMarker());syntax = "Java"; }
        else  if (ext.equals(".c") || ext.equals(".h")) {
            this.setTokenMarker(new CTokenMarker());syntax = "C"; }
        else  if (ext.equals(".cpp") || ext.equals(".cc")|| ext.equals(".hpp")|| ext.equals(".hh")) {
            this.setTokenMarker(new CCTokenMarker());syntax = "C++"; }
        else  if (ext.equals(".html") || ext.equals(".htm") || ext.equals(".xml") || ext.equals(".xsl") || ext.equals(".dtd")) {
            this.setTokenMarker(new HTMLTokenMarker());syntax = "HTML/XML"; }
        else  if (ext.equals(".spec")) {
            this.setTokenMarker(new SpecTokenMarker());syntax = "Specif"; }
        else  if (ext.equals(".sql")) {
            this.setTokenMarker(new SQLTokenMarker());syntax = "SQL"; }
        else  if (ext.equals(".php") || ext.equals(".php3") || ext.equals(".php4")|| ext.equals(".phtml")) {
            this.setTokenMarker(new PHPTokenMarker());syntax = "PHP"; }
        else  if (ext.equals(".py")) {
            this.setTokenMarker(new PythonTokenMarker());syntax = "Python"; }
        else if (currentFile.getName().toLowerCase().indexOf("makefile") != -1) {
            this.setTokenMarker(new MakefileTokenMarker());syntax = "Makefile"; }
        ext = null;
        this.repaint();
        this.requestFocus();
    }
    
    /** Retourne le fichier courant
     * @return fichier courant
     */
    public File getCurrentFile() {
        return currentFile;
    }
    
    /** Modifie le fichier courant
     * @param path nouveau fichier
     */
    public void setCurrentFile(File path) {
        currentFile = path;
    }
    
    /** Fonction permettant de definir si la zone de texte contient un fichier nouveau
     * @param newFlag boolen permettant de savoir si le fichier est nouveau
     */
    public void setNew(boolean newFlag) {
        newf = newFlag;
    }
    
    /** Teste si la zone de texte contient un fichier est nouveau
     * @return vrai si le fichier est nouveau
     */
    public boolean isNew() {
        return newf;
    }
    
    /** Teste si la zone de texte contient un fichier vide
     * @return vrai si le fichier est vide
     */
    public boolean isEmpty() {
        if (getLength() == 0)
            return true;
        else
            return false;
    }
    
    /** Teste si la zone de texte a t modifi sans tre enregistr
     * @return vrai si la zone de texte a t modifi sans tre enregistr
     */
    public boolean isDirty() {
        return dirty;
    }
    
    /** Fonction permettant de dfinir si la zone de texte a t modifi sans tre enregistr
     * @param b vrai si la zone de texte a t modifi sans tre enregistr
     */
    public void setDirty(boolean b) {
        dirty = b;
    }
    
    /** Retourne lengthtype de coloration syntaxique
     *  @return type de coloration
     */
    public String getSyntax() {
        return syntax;
    }
    
    /** Renvoie la taille du fichier courant
     * @return taille du fichier
     */
    public int getLength() {
        return getDocument().getLength();
    }
    
    public void removeUpdate(javax.swing.event.DocumentEvent documentEvent) {
        setIconDirty();
    }
    
    public void insertUpdate(javax.swing.event.DocumentEvent evt) {
        setIconDirty();
    }
    
    public void changedUpdate(javax.swing.event.DocumentEvent documentEvent) { }
    
    private void setIconDirty() {
        if (!dirty ) {
            FileTabbedPane fileTabs = this.getParentPanel().tabbedPane;
            int index = fileTabs.indexOfComponent(this.getParentPanel());
            if (index >= 0) {
                dirty = true;
                fileTabs.setIconAt(index,fileTabs.ICON_TEXT_MODIFIED);
            }
            fileTabs = null;
        }
    }
    
    public void undoableEditHappened(javax.swing.event.UndoableEditEvent e) {
        undo.addEdit(e.getEdit());
        this.getParentPanel().tabbedPane.updateUndo();
        this.getParentPanel().tabbedPane.updateRedo();
    }
    
    /** Retourne l'objet permettant de rtablir ou d'annuler une action
     * @return objet undo
     */
    public UndoManager getUndo() {
        return undo;
    }
    
    /** Renvoie l'offset du dbut de ligne d'un autre offset
     * @param off offset dont il faut chercher le dbut de ligne
     * @return offset du dbut de la ligne
     */
    public int getLineStartOffset(int off) {
        Element e = this.getDocument().getDefaultRootElement().getElement(off);
        if (e != null)
            return e.getStartOffset();
        return 0;
    }
    
    public void caretUpdate(javax.swing.event.CaretEvent evt) {
        getParentPanel().updateStatus(evt.getDot());
    }
    
    /** Fonction tire de l'diteur Jext (http://www.jext.org)
     * Return the result of a string search. Slava Pestov.
     * @param searchStr The string to be found
     * @param start The search's start offset
     * @param ignoreCase Set to true, we'll ignore the text case
     * @return True if <code>searchStr</code> has been found, false otherwise
     */
    public boolean find(String searchStr, int start, boolean ignoreCase) {
        try {
            if (searchStr.equals("") || searchStr == null) return false;
            regexp = new RE(searchStr,
            (ignoreCase == true ? RE.REG_ICASE : 0) | RE.REG_MULTILINE, new RESyntax());
            if (regexp == null) {
                getToolkit().beep();
                return false;
            }
            String text = getText(start, getLength() - start);
            REMatch match = regexp.getMatch(text);
            if (match != null) {
                this.select(start + match.getStartIndex(), start + match.getEndIndex());
                this.requestFocus();
                return true;
            }
        } catch(Exception e) { }
        return false;
    }
    
    /** Fonction tire de l'diteur Jext (http://www.jext.org)
     * Return the result of a string replace. Slava Pestov.
     * @return True if the replace has been successfully done, false otherwise
     * @param searchStr The string to be found
     * @param replaceStr The string which will replace <code>searchStr</code>
     * @param start The search's start offset
     * @param end The search's end offset
     * @param ignoreCase Set to true, we'll ignore the text case
     */
    public boolean replaceAll(String searchStr, String replaceStr, int start, int end, boolean ignoreCase) throws REException {
        boolean found = false;
        try {
            if (searchStr.equals("") || searchStr == null || replaceStr == null) return false;
            regexp = new RE(searchStr,
            (ignoreCase == true ? RE.REG_ICASE : 0) | RE.REG_MULTILINE, new RESyntax());
            if (regexp == null) {
                return false;
            }
            
            if (replaceStr == null)
                replaceStr = "";
            
            REMatch match;
            
            Element map = getDocument().getDefaultRootElement();
            int startLine = map.getElementIndex(start);
            int endLine = map.getElementIndex(end);
            
            for (int i = startLine; i <= endLine; i++) {
                Element lineElement = map.getElement(i);
                int lineStart;
                int lineEnd;
                
                if (i == startLine)
                    lineStart = start;
                else
                    lineStart = lineElement.getStartOffset();
                if (i == endLine)
                    lineEnd = end;
                else
                    lineEnd = lineElement.getEndOffset() - 1;
                
                lineEnd -= lineStart;
                String line = getText(lineStart, lineEnd);
                String newLine = regexp.substituteAll(line, replaceStr);
                if (line.equals(newLine)) continue;
                getDocument().remove(lineStart, lineEnd);
                getDocument().insertString(lineStart, newLine, null);
                
                end += (newLine.length() - lineEnd);
                found = true;
            }
        } catch(Exception e) {
            found = false;
        }
        this.requestFocus();
        return found;
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        currentFile = null;
        undo = null;
        regexp = null;
        findPattern = null;
        replacePattern = null;
        syntax = null;
        super.finalize();
    }
}
