/*
 * 20/06/2002
 *
 * DiaOptions
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.options;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import tvouriot.elife.*;
import tvouriot.elife.gui.*;

/** Fentre des options
 */
public final class DiaOptions extends JDialog {
    
    /** Onglets contenant les panneaux d'options */
    private JTabbedPane tabbedPane = new JTabbedPane();
    /** Panneau des options de l'interface */
    private PanelOptionsUserInterface userInterface;
    /** Panneau des options de l'diteur */
    private PanelOptionsEditor editor;
    
    /** Constructeur
     *  @param fr fentre principale de Elife
     */
    public DiaOptions(ElifeFrame fr) {
        
        super(fr,"Options",true);
        this.setSize(300,320);
        this.setLocation(Utilities.getCenterComponentChild(fr,this));
        
        userInterface = new PanelOptionsUserInterface(fr);
        editor = new PanelOptionsEditor(fr);
        
        JScrollPane scrollInterface = new JScrollPane(userInterface);
        scrollInterface.setBorder(null);
        tabbedPane.add("Interface",scrollInterface);
        JScrollPane scrollEditor = new JScrollPane(editor);
        scrollEditor.setBorder(null);
        tabbedPane.add("Editeur",scrollEditor);
        
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());
        
        JButton cmdAnnuler = new JButton("Annuler");
        JButton cmdSave = new JButton("Sauvegarder");
        
        JPanel pnlSouth = new JPanel();
        pnlSouth.add(cmdSave);
        pnlSouth.add(cmdAnnuler);
        
        contentPane.add(tabbedPane,BorderLayout.CENTER);
        contentPane.add(pnlSouth,BorderLayout.SOUTH);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
        
        cmdAnnuler.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                dispose();
            }
        });
        
        cmdSave.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                saveAllProperties();
                dispose();
            }
        });
    }
    
    /** Sauvegarde des nouvelles proprits
     */
    private void saveAllProperties() {
        userInterface.save();
        editor.save();
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        tabbedPane = null;
        userInterface = null;
        editor = null;
        super.finalize();
    }
}


