/*
 * 21/06/2002
 *
 * PanelOptionsUserInterface
 * Copyright (C) 2002 Vouriot Thierry
 * thierry.vouriot@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tvouriot.elife.options;

import java.awt.*;
import javax.swing.*;
import tvouriot.elife.*;
import tvouriot.elife.gui.*;

/** Panneau des options de l'interface utilisateur
 */
public final class PanelOptionsUserInterface extends AbstractPanelOptions {
    
    /** Case  cocher permettant de slectionner les "menus plats" */    
    private JCheckBox chkFlatmenus = new JCheckBox("Menus plats",false);
    /** Case  cocher permettant de rendre la barre d'outils ractive */
    private JCheckBox chkTbRollover = new JCheckBox("Barre d'outils ractive",false);
    /** <CODE>JComboBox</CODE> contenant les "look and feel" disponibles */
    private JComboBox cmbPlaf = new JComboBox();
    /** Zone de texte "X" */    
    private IntTextField txtX = new IntTextField();
    /** Zone de texte "Y" */    
    private IntTextField txtY = new IntTextField();
    /** Zone de texte "Width" */    
    private IntTextField txtWidth = new IntTextField();
    /** Zone de texte "Height" */    
    private IntTextField txtHeight = new IntTextField();
    
    /** Constructeur
     *  @param fr fentre principale de Elife
     */
    public PanelOptionsUserInterface(ElifeFrame fr) {
        
        super(fr);
       
        this.addComponent(new JLabel(" Position et taille de la fentre"));
        this.addComponent("X = ",SwingUtilities.RIGHT,txtX);
        this.addComponent("Y = ",SwingUtilities.RIGHT,txtY);
        this.addComponent("Largeur = ",SwingUtilities.RIGHT,txtWidth);
        this.addComponent("Hauteur = ",SwingUtilities.RIGHT,txtHeight);      
        this.addComponent(chkFlatmenus);
        this.addComponent(chkTbRollover);
        this.addComponent("Look and Feel :",SwingUtilities.LEFT,cmbPlaf);
        
        initPlafs();
        init();
    }
    
    /** Initilisation en fonction des proprits
     */    
    protected void init()
    {
        if (Elife.isFlatMenus())
            this.chkFlatmenus.setSelected(true); 
        if (Elife.getProperty("toolbar.rollover").equals("on"))
            this.chkTbRollover.setSelected(true); 
        
        cmbPlaf.setSelectedItem(Elife.getProperty("plaf"));
        txtX.setText(""+parent.getX());
        txtY.setText(""+parent.getY());
        txtWidth.setText(""+parent.getWidth());
        txtHeight.setText(""+parent.getHeight());
    }
    
    /** Recherche les "looks" disponibles et remplit la <CODE>JComboBox</CODE> correspondante */
    private void initPlafs() {
        UIManager.LookAndFeelInfo[] looks = UIManager.getInstalledLookAndFeels();
        for (int i=0; i<looks.length ;i++) {
            String s = looks[i].getName().toLowerCase();
            if (s.equals("metal") || s.equals("cde/motif") || s.equals("windows"))
                this.cmbPlaf.addItem(s);
        }      
    }
    
    /** Sauvegarde des nouvelles proprits
     */    
    protected void save()
    {
        Elife.setFlatMenus(chkFlatmenus.isSelected());
        Elife.setProperty("toolbar.rollover", chkTbRollover.isSelected() ? "on" : "off");
        Elife.setProperty("plaf", (String)cmbPlaf.getSelectedItem());
        int x,y,w,h;
        x = txtX.isValid() ? txtX.getValue() : parent.getX();
        y = txtY.isValid() ? txtY.getValue() : parent.getY();
        w = txtWidth.isValid() ? txtWidth.getValue() : parent.getWidth();
        h = txtHeight.isValid() ? txtHeight.getValue() : parent.getHeight();           
        parent.setBounds(x,y,w,h);
        parent.validate();
    }
    
    /** Permet de faciliter le travail du ramasse miettes
     */
    protected void finalize() throws Throwable {
        chkFlatmenus = null;
        chkTbRollover = null;
        cmbPlaf = null;
        txtX = null;
        txtY = null;
        txtWidth = null;
        txtHeight = null;
        super.finalize();
    }
}


